# -*- coding: utf-8 -*-
import datetime
import json
import mock
import os
import random

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.db.models import Model
from django.test import TestCase, TransactionTestCase

from ..models import (
    BaseModel,
    BaseStatusModel,
    Applicant,
    Check,
    Report,
    format_url
)

class TestBaseModel(BaseModel):

    class Meta:
        managed = False

    @property
    def api_get_path(self):
        return '/'


class TestBaseStatusModel(BaseStatusModel):

    class Meta:
        managed = False


class BaseModelTests(TestCase):

    def test_defaults(self):
        obj = TestBaseModel()
        self.assertEqual(obj.id, '')
        self.assertEqual(obj.created_at, None)
        self.assertEqual(obj.raw, {})

    @mock.patch.object(Model, 'save')
    def test_save(self, mock_save):
        """Test that save method returns self."""
        obj = TestBaseModel()
        self.assertEqual(obj.save(), obj)

    def test_api_get_path(self):
        """Test the api_get_path property."""
        obj = BaseModel()
        self.assertRaises(NotImplementedError, lambda: obj.api_get_path)

        obj = TestBaseModel()
        self.assertEqual(obj.api_get_path, '/')

    def test_parse_raw(self):
        """Test the parse_raw method."""
        data = {
          "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
          "created_at": "2016-10-15T19:05:50Z",
          "status": "awaiting_applicant",
          "type": "standard",
          "result": "clear",
        }
        obj = TestBaseModel(raw=data).parse_raw()
        self.assertEqual(obj.id, data['id'])
        self.assertEqual(
            obj.created_at,
            datetime.datetime(2016, 10, 15, 19, 5, 50)
        )

    @mock.patch('onfido.models.get')
    def test_fetch(self, mock_get):
        """Test the fetch method sets the raw field, but does not update."""
        obj = TestBaseModel()
        self.assertEqual(obj.fetch(), obj)
        # assert that we call the API, set the raw
        # field, but do not update the properties
        mock_get.assert_called_once()
        self.assertEqual(obj.raw, mock_get.return_value)

    @mock.patch.object(BaseModel, 'fetch')
    @mock.patch.object(BaseModel, 'parse_raw')
    @mock.patch.object(BaseModel, 'save')
    def test_pull(self, mock_save, mock_parse, mock_fetch):
        """Test the pull method sets and parses the raw field."""
        obj = TestBaseModel()
        # need to ensure that the mock returns obj so that chaining works
        mock_fetch.return_value = obj
        mock_parse.return_value = obj

        obj.pull()
        # assert that we call the API, set the raw
        # field, parse it and save the object
        mock_fetch.assert_called_once()
        mock_parse.assert_called_once()
        mock_save.assert_called_once()


class BaseStatusModelTests(TestCase):

    def test_defaults(self):
        obj = TestBaseStatusModel()
        self.assertEqual(obj.id, '')
        self.assertEqual(obj.created_at, None)
        self.assertEqual(obj.raw, {})
        self.assertEqual(obj.status, 'unknown')
        self.assertEqual(obj.result, 'unknown')
        self.assertEqual(obj.updated_at, None)
        self.assertEqual(obj.raw, {})

    def test_parse_raw(self):
        """Test the parse_raw method."""
        data = {
          "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
          "created_at": "2016-10-15T19:05:50Z",
          "status": "awaiting_applicant",
          "type": "standard",
          "result": "clear",
        }
        obj = TestBaseStatusModel(raw=data).parse_raw()
        self.assertEqual(obj.id, data['id'])
        self.assertEqual(
            obj.created_at,
            datetime.datetime(2016, 10, 15, 19, 5, 50)
        )
        self.assertEqual(obj.status, data['status'])
        self.assertEqual(obj.result, data['result'])

    @mock.patch('onfido.signals.on_status_change.send')
    @mock.patch('onfido.signals.on_completion.send')
    @mock.patch.object(BaseStatusModel, 'save')
    def test_update_status(self, mock_save, mock_complete, mock_update):
        """Test the update_status method."""
        obj = TestBaseStatusModel(status='before')
        now = datetime.datetime.now()
        self.assertEqual(obj.status, 'before')
        self.assertEqual(obj.updated_at, None)

        obj = obj.update_status('report.completed', 'after', now)
        self.assertEqual(obj.status, 'after')
        self.assertEqual(obj.updated_at, now)
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            status_before='before',
            status_after='after'
        )
        mock_complete.assert_not_called()

        # if we send 'complete' as the status we should fire
        # the second signal
        mock_update.reset_mock()
        obj = obj.update_status('report.completed', 'complete', now)
        self.assertEqual(obj.status, 'complete')
        self.assertEqual(obj.updated_at, now)
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            status_before='after',
            status_after='complete'
        )
        mock_complete.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj
        )


class ApplicantManagerTests(TestCase):

    """ApplicantManager tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)
        self.data = {
            "id": "14d2335e-4586-4ac4-aecd-b18296e7d675",
            "created_at": "2016-10-15T19:05:07Z",
        }

    @mock.patch('onfido.models.post')
    def test_create_applicant(self, mock_post):
        """Test the create method calls API and parses response."""
        mock_post.return_value = self.data
        applicant = Applicant.objects.create_applicant(user=self.user)
        mock_post.assert_called_once_with(
            format_url(Applicant.POST),
            {
                "first_name": self.user.first_name,
                "last_name": self.user.last_name,
                "email": self.user.email
            }
        )
        self.assertEqual(applicant.raw, self.data)
        self.assertEqual(applicant.id, self.data['id'])
        self.assertEqual(
            applicant.created_at,
            datetime.datetime(2016, 10, 15, 19, 5, 7)
        )

class ApplicantTests(TestCase):

    """Applicant models tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)

    def test_defaults(self):
        """Test default property values."""
        applicant = self.applicant
        self.assertEqual(applicant.id, 'foo')
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.created_at, None)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        applicant = self.applicant
        self.assertIsNotNone(str(applicant))
        self.assertIsNotNone(unicode(applicant))
        self.assertIsNotNone(repr(applicant))

    def test_api_get_path(self):
        """Test the api_get_path property."""
        applicant = self.applicant
        self.assertEqual(applicant.api_get_path, 'applicants/foo')


class CheckTests(TestCase):

    """Check models tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)
        self.check = Check(id='bar', user=self.user, applicant=self.applicant)  # noqa

    def test_defaults(self):
        """Test default property values."""
        check = Check()
        self.assertEqual(check.id, '')
        self.assertEqual(check.created_at, None)
        self.assertEqual(check.status, 'unknown')
        self.assertEqual(check.result, 'unknown')
        self.assertEqual(check.check_type, '')

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        check = self.check
        self.assertIsNotNone(str(check))
        self.assertIsNotNone(unicode(check))
        self.assertIsNotNone(repr(check))

    def test_api_get_path(self):
        """Test the api_get_path property."""
        check = self.check
        self.assertEqual(check.api_get_path, 'applicants/foo/checks/bar')

    def test_parse_raw(self):
        """Test the parse_raw method."""
        data = {
          "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
          "created_at": "2016-10-15T19:05:50Z",
          "status": "awaiting_applicant",
          "type": "standard",
          "result": "clear",
        }
        check = Check(raw=data)
        check.parse_raw()
        # real data taken from check.json
        self.assertEqual(check.id, "c26f22d5-4903-401f-8a48-7b0211d03c1f")
        self.assertEqual(
            check.created_at,
            datetime.datetime(2016, 10, 15, 19, 5, 50)
        )
        self.assertEqual(check.status, "awaiting_applicant")
        self.assertEqual(check.result, "clear")


class ReportTests(TestCase):

    """Report models tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)
        self.check = Check(id='bar', user=self.user, applicant=self.applicant)  # noqa
        self.report = Report(id='baz', user=self.user, onfido_check=self.check)  # noqa

    def test_defaults(self):
        """Test default property values."""
        report = self.report
        self.assertEqual(report.id, 'baz')
        self.assertEqual(report.created_at, None)
        self.assertEqual(report.status, 'unknown')
        self.assertEqual(report.result, 'unknown')
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.report_type, '')

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        report = self.report
        self.assertIsNotNone(str(report))
        self.assertIsNotNone(unicode(report))
        self.assertIsNotNone(repr(report))

    def test_api_get_path(self):
        """Test the api_get_path property."""
        report = self.report
        self.assertEqual(report.api_get_path, 'checks/bar/reports/baz')

    def test_parse_raw(self):
        """Test the parse_raw method."""
        # with open(os.path.join(os.path.dirname(__file__), 'test_files/check.json')) as f:
        #     data = json.load(f)
        data = {
          "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
          "created_at": "2016-10-15T19:05:50Z",
          "status": "awaiting_applicant",
          "result": "clear",
          "name": "identity",
        }
        report = Report(raw=data)
        report.parse_raw()
        # real data taken from check.json
        self.assertEqual(report.id, "c26f22d5-4903-401f-8a48-7b0211d03c1f")
        self.assertEqual(
            report.created_at,
            datetime.datetime(2016, 10, 15, 19, 5, 50)
        )
        self.assertEqual(report.status, "awaiting_applicant")
        self.assertEqual(report.result, "clear")
        self.assertEqual(report.report_type, "identity")



# -*- coding: utf-8 -*-
import json
import mock

from django.test import TestCase, Client, RequestFactory

from ..models import Check, Report
from ..views import (
    status_update,
    _update_status,
    _get_manager
)

class ViewTests(TestCase):

    """Test for views module."""

    def test__get_manager(self):
        """Test the _get_manager function."""
        self.assertEqual(_get_manager('report'), Report.objects)
        self.assertEqual(_get_manager('check'), Check.objects)
        self.assertRaises(AssertionError, _get_manager, 'foo')

    def test__update_status(self):
        """Test the _status_update function."""
        # now mock out a real object, and check we're calling update_status
        with mock.patch.object(Check, 'objects') as mock_manager:
            obj = mock.Mock()
            mock_manager.get.return_value = obj
            response = _update_status(
                "check", 'obj_id', 'action', 'status', 'completed_at'
            )
            obj.update_status.assert_called_once_with(
                'status', 'completed_at'
            )

    def test_status_update(self):
        """Test the status_update view function."""
        data = {
            "payload": {
                "resource_type": "check",
                "action": "check.form_completed",
                "object": {
                    "id": "5345badd-f4bf-4240-9f3b-ffb998bda09e",
                    "status": "in_progress",
                    "completed_at": "2016-10-15 11:34:09 UTC",
                    "href": "https://api.onfido.com/v1/applicants/4d390bbd-63c7-4960-8304-a7a04a8051e8/checks/5345badd-f4bf-4240-9f3b-ffb998bda09e"
                }
            }
        }
        factory = RequestFactory()

        def assert_update(data, message):
            request = factory.post('/', data=json.dumps(data), content_type='application/json')
            response = status_update(request)
            self.assertEqual(response.status_code, 200)
            self.assertEqual(response.content, message)

        # invalid payload
        assert_update({}, 'Unexpected event content.')

        # invalid resource type
        data['payload']['resource_type'] = "foo"
        assert_update(data, 'Unknown resource type.')

        # unknown Check
        data['payload']['resource_type'] = "check"
        with mock.patch.object(Check, 'objects') as mock_manager:
            mock_manager.get.side_effect = Check.DoesNotExist()
            assert_update(data, 'Check not found.')

        # unknown Report
        data['payload']['resource_type'] = "report"
        with mock.patch.object(Report, 'objects') as mock_manager:
            mock_manager.get.side_effect = Report.DoesNotExist()
            assert_update(data, 'Report not found.')

        # unknown exception
        with mock.patch('onfido.views._status_update') as mock_update:
            mock_update.side_effect = Exception("foobar")
            assert_update(data, 'Unknown error.')

        # valid payload / object
        with mock.patch('onfido.views._status_update') as mock_manager:
            request = factory.post('/', data=json.dumps(data), content_type='application/json')
            assert_update(data, 'Update processed.')



# -*- coding: utf-8 -*-
import simplejson as json  # simplejson supports Decimal

from django.contrib import admin
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from .models import Applicant, Check, Report


def pprint(data):
    """
    Return an indented HTML pretty-print version of JSON.

    Take the event_payload JSON, indent it, order the keys and then
    present it as a <code> block. That's about as good as we can get
    until someone builds a custom syntax function.

    """
    pretty = json.dumps(
        data,
        sort_keys=True,
        indent=4,
        separators=(',', ': ')
    )
    return mark_safe("<code>%s</code>" % pretty.replace(" ", "&nbsp;"))


class ApplicantAdmin(admin.ModelAdmin):

    """Admin model for Applicant objects."""

    list_display = ('id', 'user', 'created_at')
    list_filter = ('created_at',)
    ordering = ('user__first_name', 'user__last_name')
    readonly_fields = ('id', 'created_at', '_raw')
    raw_id_fields = ('user',)
    exclude = ('raw',)

    def _raw(self, obj):
        """Return pprint version of the raw field."""
        return pprint(obj.raw)
    _raw.short_description = _("Raw")

admin.site.register(Applicant, ApplicantAdmin)


class CheckAdmin(admin.ModelAdmin):

    """Admin model for Check objects."""

    list_display = ('id', 'applicant', 'check_type', 'created_at')
    list_filter = ('check_type', 'created_at',)
    readonly_fields = ('created_at', 'id')
    raw_id_fields = ('applicant',)

admin.site.register(Check, CheckAdmin)


class ReportAdmin(admin.ModelAdmin):

    """Admin model for Report objects."""

    list_display = ('onfido_check', 'report_type', 'created_at')
    list_filter = ('created_at', 'report_type')
    readonly_fields = ('created_at', 'id')
    raw_id_fields = ('onfido_check',)

admin.site.register(Report, ReportAdmin)



# -*- coding: utf-8 -*-
import datetime
import logging

from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext as _

from .api import get, post, url as format_url
from .db.fields import JSONField
from .signals import on_status_change, on_completion

logger = logging.getLogger(__name__)


class BaseModel(models.Model):

    """Base model used to set timestamps."""

    id = models.CharField(
        primary_key=True,
        max_length=40,
        help_text=_("The id returned from the Onfido API."),
    )
    created_at = models.DateTimeField(
        help_text=_("The timestamp returned from the Onfido API.")
    )
    raw = JSONField(
        help_text=_("The raw JSON returned from the API.")
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        """Save object and return self (for chaining methods)."""
        super(BaseModel, self).save(*args, **kwargs)
        return self

    @property
    def api_get_path(self):
        """Return relative API path that points to the GET endpoint."""
        raise NotImplementedError()

    def parse_raw(self):
        """Parses the raw value out into other properties."""
        self.id = self.raw['id']
        self.created_at = datetime.datetime.strptime(
            self.raw['created_at'],
            '%Y-%m-%dT%H:%M:%SZ'
        )
        return self

    def fetch(self):
        """Fetch latest version from Api and update local data."""
        self.raw = get(format_url(self.api_get_path))
        return self

    def pull(self):
        """Fetch and parse the remote data."""
        return self.fetch().parse_raw().save()

class BaseStatusModel(BaseModel):

    """Base class for models with a status and result field."""

    status = models.CharField(
        max_length=20,
        default='unknown',
        help_text=_("The current state of the check / report (from API).")
    )
    result = models.CharField(
        max_length=20,
        help_text=_("The final result of the check / reports (from API)."),
        default='unknown'
    )
    updated_at = models.DateTimeField(
        blank=True, null=True,
        help_text=_("The timestamp of the most recent status change.")
    )

    class Meta:
        abstract = True

    def update_status(self, event, new_status, timestamp):
        """
        Update the status field of the object and fire signal(s).

        When the app receives an event callback from Onfido, we update
        the status of the relevant Check/Report, and then fire the
        signals that allow external apps to hook in to this event.

        If the update is a change to 'complete', then we fire a second
        signal - 'complete' is the terminal state change, and therefore
        of most interest to clients - typically the on_status_update
        signal would be registered for logging a complete history of
        changes, whereas the on_completion signal would be used to do
        something more useful - updating the status of the user, sending
        them an email etc.

        Args:
            event: string, the name of the event, e.g. 'report.completed'
            new_status: string, the new status, e.g. 'complete'
            timestamp: DateTime, from the callback `completed_at` value

        Returns the updated object.

        """
        # swap statuses around so we record old / new
        self.status, old_status = new_status, self.status
        self.updated_at = timestamp
        self.save()
        on_status_change.send(
            self.__class__,
            instance=self,
            status_before=old_status,
            status_after=new_status
        )
        if new_status == 'complete':
            on_completion.send(self.__class__, instance=self)
        return self

    def parse_raw(self):
        """Parses the raw value out into other properties."""
        super(BaseStatusModel, self).parse_raw()
        self.result=self.raw['result']
        self.status=self.raw['status']
        return self


class ApplicantManager(models.Manager):

    """Custom Applicant queryset."""

    def create_applicant(self, user):
        """Create a new applicant in Onfido from a user."""
        data = {
            "first_name": user.first_name,
            "last_name": user.last_name,
            "email": user.email
        }
        response = post(format_url(Applicant.POST), data)
        logger.debug(response)
        return Applicant(user=user, raw=response).parse_raw().save()


class Applicant(BaseModel):

    """An Onfido applicant record."""

    GET = 'applicants/{}'
    POST = 'applicants'

    user = models.ForeignKey(
        User,
        help_text=_("Django user that maps to this applicant.")
    )

    objects = ApplicantManager()

    def __unicode__(self):
        return self.user.get_full_name() or self.user.username

    def __repr__(self):
        return u"<Applicant id=%s user='%s'>" % (
            self.id, self.user.username
        )

    @property
    def api_get_path(self):
        return Applicant.GET.format(self.id)


def create_check(
    applicant, check_type, reports,
    suppress_form_emails=True,
    redirect_uri=None
):
    """
    Create a new Check (and child Reports).

    Args:
        applicant: Applicant for whom the checks are being made.
        check_type: string, currently only 'standard' is supported.
        reports: list of strings, each of which is a valid report type.

    Kwargs:
        suppress_form_emails: bool, if True then suppress the email the Onfido
            itself would normally send when a check is initiated. Defaults to
            True.
        redirect_uri: string, a url to which to direct the user _after_ they
            have submitted all the information requested.

    Returns a new Check object, and creates the child Report objects.

    """
    data = {
        "type": check_type,
        "suppress_form_emails": suppress_form_emails,
        "reports": [{'name': r for r in reports}],
        "redirect_uri": redirect_uri
    }
    response = post(format_url(Applicant.POST), data)
    logger.debug(response)
    return Check(applicant=applicant, raw=response).parse_raw().save()


class Check(BaseStatusModel):

    """The state of an individual check made against an Applicant."""

    GET = 'applicants/{}/checks/{}'
    POST = 'applicants/{}/checks'

    CHECK_TYPE_CHOICES = (
        ('express', 'Express check'),
        ('standard', 'Standard check')
    )
    # DEPRECATED, pls keep for reference
    CHECK_STATUS_CHOICES = (
        ('unknown', 'Unknown'),
        ('in_progress', 'In progress'),
        ('awaiting_applicant', 'Awaiting applicant'),
        ('complete', 'Complete'),
        ('withdrawn', 'Withdrawn'),
        ('paused', 'Paused'),
        ('reopened', 'Reopened'),
    )
    # DEPRECATED, pls keep for reference
    CHECK_RESULT_CHOICES = (
        ('clear', 'Clear'),
        ('consider', 'Consider')
    )
    user = models.ForeignKey(
        User,
        help_text=_("The Django user (denormalised from Applicant to make navigation easier)."),  # noqa
        related_name='onfido_checks'
    )
    applicant = models.ForeignKey(
        Applicant,
        help_text=_("The applicant for whom the check is being made."),
        related_name='checks'
    )
    check_type = models.CharField(
        max_length=10,
        choices=CHECK_TYPE_CHOICES,
        help_text=_("See https://documentation.onfido.com/#check-types")
    )

    def __unicode__(self):
        return (
            u"%s for %s" % (
                self.get_check_type_display().capitalize(),
                self.user.get_full_name() or self.user.username
            )
        )

    def __repr__(self):
        return (
            u"<Check id=%s type='%s' user='%s'>" % (
                self.id,
                self.check_type,
                self.user
            )
        )

    @property
    def api_get_path(self):
        return Check.GET.format(self.applicant.id, self.id)

    def parse_raw(self):
        """Parses the raw value out into other properties."""
        super(Check, self).parse_raw()
        self.check_type=self.raw['type']
        return self


class Report(BaseStatusModel):

    """Specific reports associated with a Check."""

    REPORT_TYPE_CHOICES = (
        ('unknown', '-'),
        ('identity', 'Identity report'),
        ('document', 'Document report'),
        ('street_level', 'Street level report'),
        ('facial_similarity', 'Facial similarity report'),
    )
    # DEPRECATED, pls keep for reference
    REPORT_STATUS_CHOICES = (
        ('unknown', 'Unknown'),
        ('awaiting_data', 'Awaiting data'),
        ('awaiting_approval', 'Awaiting approval'),
        ('complete', 'Complete'),
        ('withdrawn', 'Withdrawn'),
        ('paused', 'Paused'),
        ('cancelled', 'Cancelled'),
    )
    # DEPRECATED, pls keep for reference
    REPORT_RESULT_CHOICES = (
        ('unknown', 'Unknown'),
        ('clear', 'Clear'),
        ('consider', 'Consider'),
        ('unidentified', 'Unidentified'),
    )
    user = models.ForeignKey(
        User,
        help_text=_("The Django user (denormalised from Applicant to make navigation easier)."),  # noqa
        related_name='onfido_reports'
    )
    onfido_check = models.ForeignKey(
        Check,
        help_text=_("Check to which this report is attached."),
        related_name='reports'
    )
    report_type = models.CharField(
        max_length=20,
        choices=REPORT_TYPE_CHOICES,
        help_text=_("The name of the report - see https://documentation.onfido.com/#reports")
    )

    def __unicode__(self):
        return (
            u"%s for %s" % (
                self.get_report_type_display().capitalize(),
                self.user.get_full_name() or self.user.username
            )
        )

    def __repr__(self):
        return (
            u'<Report id=%s type=%s user=%s>' % (
                self.id,
                self.report_type,
                self.user
            )
        )

    @property
    def api_get_path(self):
        return 'checks/{}/reports/{}'.format(self.onfido_check.id, self.id)

    def parse_raw(self):
        """Parses the raw value out into other properties."""
        super(Report, self).parse_raw()
        self.report_type=self.raw['name']
        return self



# -*- coding: utf-8 -*-
"""onfido views.

The Onfido views are mainly callback handlers for the updates
in status' of checks and reports.

See https://documentation.onfido.com/?shell#webhooks

"""
import json
import logging

from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse

from .models import Check, Report

logger = logging.getLogger(__name__)


def status_update(request):
    """
    Handle event callbacks from the API.

    This is the request handler which does little other than
    log the request and call the _webhook function which does
    the processing. Done like this to make it easier to test
    without having to set up request objects.

    NB This view function will always return a 200 status -
    in order to prevent Onfido from endlessly retrying.

    """
    logger.debug("Received Onfido callback: {}".format(request.body))
    data = json.loads(request.body)
    try:
        _update_status(
            data['payload']['resource_type'],
            data['payload']['object']['id'],
            data['payload']['action'],
            data['payload']['object']['status'],
            data['payload']['object']['completed_at']
        )
        return HttpResponse("Update processed.")
    except KeyError:
        logger.warn("Missing Onfido event content: {}".format(data))
        return HttpResponse("Unexpected event content.")
    except AssertionError as ex:
        logger.warn("Unknown Onfido resource type")
        return HttpResponse("Unknown resource type.")
    except Check.DoesNotExist:
        logger.warn("Onfido check does not exist")
        return HttpResponse("Check not found.")
    except Report.DoesNotExist:
        logger.warn("Onfido report does not exist")
        return HttpResponse("Report not found.")
    except Exception:
        logger.exception("Onfido update could not be processed.")
        return HttpResponse("Unknown error.")


def _get_manager(resource_type):
    """Get appropriate model manager for a resource."""
    assert resource_type in ('report', 'check')
    return (
        Report.objects if resource_type == 'report'
        else Check.objects
    )


def _update_status(resource_type, obj_id, action, status, completed_at):
    """Process status update receive via webhook."""
    logger.debug("Processing '%s' action on %s.%s", action, resource_type, obj_id)
    manager = _get_manager(resource_type)
    obj = manager.get(id=obj_id)
    obj.update_status(status, completed_at)



# -*- coding: utf-8 -*-
"""onfido settings."""
from os import getenv

from django.conf import settings

# read the api key in from settings, fall back to environment
ONFIDO_API_KEY = getattr(settings, 'ONFIDO_API_KEY', None) or getenv('ONFIDO_API_KEY')

# the API HTTP root url
ONFIDO_API_ROOT = "https://api.onfido.com/v2/"
